# Python implementation by Dustin Zacharias (2017), method based on Fabrice Rouah (volopta.com)

# HN GARCH Price

def HNP(prices,V, S, K, r, T, PutCall):		#PutCall=1 -> Call

    output = main()

    return HNC(output[0],output[1],output[2],output[3],output[4],V, S, K, r, T, PutCall)

