# Python implementation by Dustin Zacharias (2017), method based on Fabrice Rouah (volopta.com)

from setuptools import setup

setup(name='hngoption',
      version='0.1',
      description='Heston Nandi GARCH Option Pricing Model (2000)',
      url='https://github.com/SW71X/HNGoption/archive/1.tar.gz',
      author='Dustin L. Zacharias',
      author_email='dustinl.zacharias@gmail.com',
      license='MIT',
      packages=['hngoption'],
      zip_safe=False)